@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2005-2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2005-2013 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 /**
  * @file src/identity/gnunet-identity-gtk_about.c
@@ -31,7 +31,7 @@
  * This displays an about window
  */
 void
-GNUNET_GTK_conversation_about_imagemenuitem_activate_cb (GtkWidget * dummy,
+gnunet_conversation_gtk_about_imagemenuitem_activate_cb (GtkWidget * dummy,
                                                          gpointer data)
 {
   GNUNET_GTK_display_about ("gnunet_conversation_gtk_about_window.glade");
