@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2010 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2010 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -110,7 +110,7 @@ struct SearchTab
   /**
    * Animation handle associated with the tree store.
    */
-  struct GNUNET_FS_AnimationTreeViewHandle *atv;
+  struct GNUNET_GTK_AnimationTreeViewHandle *atv;
 
   /**
    * Number of results we got for this search.
@@ -169,7 +169,7 @@ struct DownloadEntry
   int is_recursive;
 
   /**
-   * Is this a directory? (GNUNET_SYSERR is used for "maybe"!)
+   * Is this a directory? (#GNUNET_SYSERR is used for "maybe"!)
    */
   int is_directory;
 
