@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2011, 2012, 2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2011, 2012, 2013 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -34,6 +34,21 @@
  */
 #define LOOKUP_TIMEOUT GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_MINUTES, 2)
 
+
+/**
+ * Emits a beep using the display of the screen of the main file sharing window.
+ */
+static void
+beep ()
+{
+  gdk_display_beep
+    (gdk_screen_get_display
+     (gdk_window_get_screen
+      (gtk_widget_get_parent_window
+       (GTK_WIDGET (GNUNET_FS_GTK_get_main_window_object ("GNUNET_GTK_main_window"))))));
+}
+
+
 /**
  * Start a search.
  *
@@ -64,10 +79,10 @@ abort_search_lookup (struct SearchLookup *sl)
   GNUNET_CONTAINER_DLL_remove (main_ctx->sl_head,
 			       main_ctx->sl_tail,
 			       sl);
-  if (GNUNET_SCHEDULER_NO_TASK != sl->timeout_task)
+  if (NULL != sl->timeout_task)
   {
     GNUNET_SCHEDULER_cancel (sl->timeout_task);
-    sl->timeout_task = GNUNET_SCHEDULER_NO_TASK;
+    sl->timeout_task = NULL;
   }
   if (NULL != sl->gns)
   {
@@ -84,15 +99,13 @@ abort_search_lookup (struct SearchLookup *sl)
  * the GNS namespace times out.
  *
  * @param cls the 'struct SearchLookup'
- * @param tc scheduler context
  */
 static void
-timeout_search_lookup (void *cls,
-		       const struct GNUNET_SCHEDULER_TaskContext *tc)
+timeout_search_lookup (void *cls)
 {
   struct SearchLookup *sl = cls;
 
-  sl->timeout_task = GNUNET_SCHEDULER_NO_TASK;
+  sl->timeout_task = NULL;
   GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
 	      _("Failed to resolve namespace in time\n"));
   abort_search_lookup (sl);
@@ -232,7 +245,6 @@ start_search (struct GNUNET_GTK_MainWindowContext *main_ctx)
 				 &pub_sks_zone,
 				 GNUNET_GNSRECORD_TYPE_PKEY,
 				 GNUNET_NO,
-				 NULL /* no shortening */,
 				 &handle_gns_result,
 				 sl);
     GNUNET_CONTAINER_DLL_insert (main_ctx->sl_head,
@@ -302,11 +314,9 @@ main_window_search_entry_key_press_event_cb (GtkWidget * widget,
  * Asynchronously clean up PseuLookupContext.
  *
  * @param cls the `struct PseuLookupContext` to clean up.
- * @param tc scheduler context
  */
 static void
-end_pseu_lookup (void *cls,
-		 const struct GNUNET_SCHEDULER_TaskContext *tc)
+end_pseu_lookup (void *cls)
 {
   struct PseuLookupContext *lctx = cls;
 
@@ -384,7 +394,7 @@ store_continuation (void *cls,
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
 		_("Failed to save record: %s\n"),
 		emsg);
-    gdk_beep ();
+    beep ();
   }
   abort_pseu_lookup (lctx);
 }
@@ -614,7 +624,7 @@ GNUNET_GTK_enter_nick_window_delete_event_cb (GtkWidget *widget,
 					      void *cls)
 {
   /* Don't allow GTK to kill the window, user must click execute or cancel */
-  gdk_beep ();
+  beep ();
   return TRUE;
 }
 
@@ -633,7 +643,7 @@ GNUNET_FS_GTK_pseu_progress_dialog_delete_event_cb (GtkWidget *widget,
 						    void *cls)
 {
   /* Don't allow GTK to kill the window, until the search is finished */
-  gdk_beep ();
+  beep ();
   return TRUE;
 }
 
@@ -695,11 +705,10 @@ GNUNET_FS_GTK_save_button_clicked_cb (GtkButton * button,
     /* show the window */
     gtk_window_present (GTK_WINDOW (lctx->progress_dialog));
     lctx->lr = GNUNET_GNS_lookup (main_ctx->gns,
-				  GNUNET_GNS_MASTERZONE_STR,
+				  GNUNET_GNS_EMPTY_LABEL_AT,
 				  &pkey,
 				  GNUNET_GNSRECORD_TYPE_NICK,
 				  GNUNET_NO,
-				  NULL,
 				  &lookup_finished,
 				  lctx);
   }
