@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2005-2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2005-2013 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -384,7 +384,7 @@ struct AddDirClientContext
   /**
    * Task scheduled to stop the scanner on errors.
    */
-  GNUNET_SCHEDULER_TaskIdentifier kill_task;
+  struct GNUNET_SCHEDULER_Task * kill_task;
 
   /**
    * Default options to use for sharing when adding files during the scan.
@@ -557,8 +557,8 @@ update_selectivity_edit (struct MainPublishingDialogContext *ctx)
  * @param ts the changed selection
  * @param user_data master publishing dialog context of our window
  */
-static void
-selection_changed_cb (GtkTreeSelection * ts,
+void
+GNUNET_GTK_master_publish_dialog_file_informatino_treeview_selection_changed_cb (GtkTreeSelection * ts,
 		      gpointer user_data)
 {
   struct MainPublishingDialogContext *ctx = user_data;
@@ -899,8 +899,9 @@ GNUNET_GTK_master_publish_dialog_identifier_entry_changed_cb (GtkWidget *widget,
  * @param ts the changed selection
  * @param user_data master publishing dialog context of our window
  */
-static void
-identifiers_selection_changed_cb (GtkTreeSelection *ts, gpointer user_data)
+void
+GNUNET_GTK_master_publish_dialog_previous_identifiers_treeview_selection_changed_cb (GtkTreeSelection *ts,
+                                                                                     gpointer user_data)
 {
   struct MainPublishingDialogContext *ctx = user_data;
   GtkTreeIter iter;
@@ -1257,10 +1258,10 @@ static void
 destroy_progress_dialog (struct AddDirClientContext *adcc)
 {
   GNUNET_assert (NULL == adcc->ds);
-  if (GNUNET_SCHEDULER_NO_TASK != adcc->kill_task)
+  if (NULL != adcc->kill_task)
   {
     GNUNET_SCHEDULER_cancel (adcc->kill_task);
-    adcc->kill_task = GNUNET_SCHEDULER_NO_TASK;
+    adcc->kill_task = NULL;
   }
   gtk_widget_destroy (adcc->progress_dialog);
   g_object_unref (G_OBJECT (adcc->progress_dialog_builder));
@@ -1456,15 +1457,13 @@ add_share_items_to_treestore (struct AddDirClientContext *adcc,
  * in a separate task).
  *
  * @param cls  progress dialog context of our window
- * @param tc scheduler context, unused
  */
 static void
-stop_scanner_task (void *cls,
-		   const struct GNUNET_SCHEDULER_TaskContext *tc)
+stop_scanner_task (void *cls)
 {
   struct AddDirClientContext *adcc = cls;
 
-  adcc->kill_task = GNUNET_SCHEDULER_NO_TASK;
+  adcc->kill_task = NULL;
   if (NULL != adcc->ds)
     {
       GNUNET_FS_directory_scan_abort (adcc->ds);
@@ -1561,7 +1560,7 @@ directory_scan_cb (void *cls,
     break;
   case GNUNET_FS_DIRSCANNER_INTERNAL_ERROR:
     if ( (NULL != adcc->ds) &&
-	 (GNUNET_SCHEDULER_NO_TASK == adcc->kill_task) )
+	 (NULL == adcc->kill_task) )
       {
 	insert_progress_dialog_text (adcc, _("Operation failed (press cancel)\n"));
 	adcc->kill_task = GNUNET_SCHEDULER_add_now (&stop_scanner_task, adcc);
@@ -1607,7 +1606,7 @@ scan_file_or_directory (struct MainPublishingDialogContext *ctx,
 {
   struct AddDirClientContext *adcc;
 
-  adcc = GNUNET_malloc (sizeof (struct AddDirClientContext));
+  adcc = GNUNET_new (struct AddDirClientContext);
   adcc->ctx = ctx;
   GNUNET_CONTAINER_DLL_insert_tail (ctx->adddir_head, ctx->adddir_tail, adcc);
   adcc->directory_scan_bo = *bo;
@@ -1921,7 +1920,7 @@ update_treeview_after_edit (void *cls, struct GNUNET_FS_FileInformation *fi,
                       PUBLISH_MC_REPLICATION_LEVEL,
                       (guint) bo->replication_level,
                       -1);
-  GNUNET_free (name);
+  GNUNET_free_non_null (name);
   return GNUNET_SYSERR;
 }
 
@@ -1960,7 +1959,7 @@ GNUNET_GTK_master_publish_dialog_edit_button_clicked_cb (GtkWidget * dummy,
   struct MainPublishingDialogContext *ctx = user_data;
   struct EditPublishContext *epc;
 
-  epc = GNUNET_malloc (sizeof (struct EditPublishContext));
+  epc = GNUNET_new (struct EditPublishContext);
   epc->tm = ctx->file_info_treemodel;
   if (! gtk_tree_selection_get_selected (ctx->file_info_selection, NULL, &epc->iter))
   {
@@ -2835,12 +2834,6 @@ GNUNET_GTK_main_menu_file_publish_activate_cb (GtkWidget * dummy,
 
   g_signal_connect (ctx->identifiers_expander, "notify::expanded", G_CALLBACK (expander_callback), ctx);
 
-  /* connect signals; FIXME-GTK3: these could be connected with (modern) Glade */
-  g_signal_connect (G_OBJECT (ctx->file_info_selection), "changed",
-                    G_CALLBACK (selection_changed_cb), ctx);
-  g_signal_connect (G_OBJECT (ctx->identifiers_selection), "changed",
-                    G_CALLBACK (identifiers_selection_changed_cb), ctx);
-
   cfg = GNUNET_FS_GTK_get_configuration ();
   ctx->identity = GNUNET_IDENTITY_connect (cfg,
 					   &add_namespace_to_ts,
