@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2005-2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2005-2013 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -293,9 +293,11 @@ static void
 change_metatypes (struct EditPublicationDialogContext *ctx, gint code)
 {
   gint pubtype_count;
+#if HAVE_EXTRACTOR
   gint max_type;
   gint i;
   GtkTreeIter iter;
+#endif
 
   /* double-check that 'code' is valid */
   for (pubtype_count = 0; NULL != types[pubtype_count]; pubtype_count++) ;
@@ -303,6 +305,7 @@ change_metatypes (struct EditPublicationDialogContext *ctx, gint code)
 
   /* clear existing selection of metatypes */
   gtk_list_store_clear (ctx->metatypes_liststore);
+#if HAVE_EXTRACTOR
   max_type = EXTRACTOR_metatype_get_max ();
   /* add new types based on selection */
   for (i = 0; types[code][i] != EXTRACTOR_METATYPE_RESERVED; i++)
@@ -320,6 +323,7 @@ change_metatypes (struct EditPublicationDialogContext *ctx, gint code)
                                          EXTRACTOR_metatype_to_description
                                          (types[code][i]),
                                          -1);
+#endif
 }
 
 
@@ -353,9 +357,8 @@ GNUNET_GTK_edit_publication_type_combo_changed_cb (GtkComboBox * widget,
  * @param ts the tree selection object
  * @param user_data the 'struct EditPublicationDialogContext'
  */
-/* FIXME-GTK3: connect this signal via glade (modern versions of Glade support this) */
-static void
-metadata_selection_changed_cb (GtkTreeSelection *ts,
+void
+GNUNET_GTK_edit_publication_metadata_treeview_selection_changed_cb (GtkTreeSelection *ts,
 			       gpointer user_data)
 {
   struct EditPublicationDialogContext *ctx = user_data;
@@ -404,10 +407,12 @@ GNUNET_GTK_edit_publication_metadata_tree_view_type_renderer_edited_cb (GtkCellR
   gtk_list_store_set (ctx->meta_liststore, &iter,
                       PUBLISH_METADATA_MC_TYPE, type_id,
                       PUBLISH_METADATA_MC_FORMAT, EXTRACTOR_METAFORMAT_UTF8,
+#if HAVE_EXTRACTOR
                       PUBLISH_METADATA_MC_TYPE_NAME,
                       EXTRACTOR_metatype_to_string (type_id),
                       PUBLISH_METADATA_MC_DESCRIPTION,
                       EXTRACTOR_metatype_to_description (type_id),
+#endif
                       -1);
 }
 
@@ -607,9 +612,8 @@ GNUNET_GTK_edit_publication_metadata_preview_file_chooser_button_file_set_cb (Gt
  * @param ts the tree selection object
  * @param user_data the 'struct EditPublicationDialogContext'
  */
-/* FIXME-GTK3: connect this signal via glade (modern versions of Glade support this) */
-static void
-keywords_selection_changed_cb (GtkTreeSelection *ts,
+void
+GNUNET_GTK_edit_publication_keyword_list_treeview_selection_changed_cb (GtkTreeSelection *ts,
 			       gpointer user_data)
 {
   struct EditPublicationDialogContext *ctx = user_data;
@@ -874,6 +878,7 @@ preserve_meta_items (void *cls, const char *plugin_name,
 }
 
 
+#if HAVE_EXTRACTOR
 /**
  * Type of a function that libextractor calls for each
  * meta data item found.  Used to get the mime type.
@@ -910,6 +915,7 @@ le_callback (void *cls,
   }
   return 0;
 }
+#endif
 
 
 /**
@@ -922,16 +928,26 @@ static char *
 get_mime_type (const void *data,
 	       gsize data_size)
 {
+#if HAVE_EXTRACTOR
   char *ret;
   struct EXTRACTOR_PluginList *pl;
 
-  pl = EXTRACTOR_plugin_add (NULL, "mime", NULL, EXTRACTOR_OPTION_DEFAULT_POLICY);
+  pl = EXTRACTOR_plugin_add (NULL,
+                             "mime",
+                             NULL,
+                             EXTRACTOR_OPTION_DEFAULT_POLICY);
   if (NULL == pl)
     return NULL;
   ret = NULL;
-  EXTRACTOR_extract (pl, NULL, data, data_size, &le_callback, &ret);
+  EXTRACTOR_extract (pl,
+                     NULL,
+                     data, data_size,
+                     &le_callback, &ret);
   EXTRACTOR_plugin_remove_all (pl);
   return ret;
+#else
+  return NULL;
+#endif
 }
 
 
@@ -947,7 +963,7 @@ get_mime_type (const void *data,
  * @param bo block options (can be modified)
  * @param do_index should we index (can be modified)
  * @param client_info pointer to client context set upon creation (can be modified)
- * @return GNUNET_SYSERR (aborts after first call)
+ * @return #GNUNET_SYSERR (aborts after first call)
  */
 static int
 file_information_update (void *cls, struct GNUNET_FS_FileInformation *fi,
@@ -1057,7 +1073,7 @@ file_information_update (void *cls, struct GNUNET_FS_FileInformation *fi,
 	/* fall back to Gtk mime-detection; this only works on GNU */
 	GFileInfo *finfo;
 	const char *gmime;
-	
+
 	finfo = g_file_query_info (f, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
 				   0, NULL, NULL);
 	gmime = g_file_info_get_attribute_string (finfo,
@@ -1241,11 +1257,11 @@ file_information_import (void *cls,
  *
  * @param parent parent window of the dialog
  * @param fip information about the file information that is to be edited
- * @param is_namespace_edit GNUNET_YES if we are editing a namespace advertisement;
+ * @param is_namespace_edit #GNUNET_YES if we are editing a namespace advertisement;
  *        this means that keywords are required and that a "root" can be
  *        entered
  * @param cb function to call when the dialog is closed
- * @param cb_cls closure for 'cb'
+ * @param cb_cls closure for @a cb
  */
 void
 GNUNET_FS_GTK_edit_publish_dialog (GtkWindow * parent,
@@ -1260,10 +1276,8 @@ GNUNET_FS_GTK_edit_publish_dialog (GtkWindow * parent,
   GtkLabel *index_label;
   GtkLabel *root_label;
   struct EditPublicationDialogContext *ctx;
-  GtkTreeSelection *meta_selection;
-  GtkTreeSelection *keywords_selection;
 
-  ctx = GNUNET_malloc (sizeof (struct EditPublicationDialogContext));
+  ctx = GNUNET_new (struct EditPublicationDialogContext);
   ctx->fip = fip;
   ctx->preview_changed = GNUNET_NO;
   ctx->allow_no_keywords = is_namespace_edit ? GNUNET_NO : GNUNET_YES;
@@ -1273,7 +1287,7 @@ GNUNET_FS_GTK_edit_publish_dialog (GtkWindow * parent,
   ctx->meta_combo_selected_type_id = -1;
   ctx->builder = GNUNET_GTK_get_new_builder ("gnunet_fs_gtk_edit_publication.glade", ctx);
 
-  if (ctx->builder == NULL)
+  if (NULL == ctx->builder)
   {
     GNUNET_free (ctx);
     return;
@@ -1365,14 +1379,6 @@ GNUNET_FS_GTK_edit_publish_dialog (GtkWindow * parent,
   gtk_widget_set_visible (GTK_WIDGET (root_label),
 			  is_namespace_edit ? TRUE : FALSE);
 
-  /* FIXME-GTK3: these signal handlers can be set by (modern) versions of Glade */
-  keywords_selection = gtk_tree_view_get_selection (ctx->keywords_treeview);
-  g_signal_connect (G_OBJECT (keywords_selection), "changed",
-                    G_CALLBACK (keywords_selection_changed_cb), ctx);
-  meta_selection = gtk_tree_view_get_selection (ctx->meta_treeview);
-  g_signal_connect (G_OBJECT (meta_selection), "changed",
-                    G_CALLBACK (metadata_selection_changed_cb), ctx);
-
   /* import meta data and options */
   GNUNET_FS_file_information_inspect (fip, &file_information_import, ctx);
 
