@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2012 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2012 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -214,7 +214,7 @@ GNUNET_FS_GTK_unindex_button_clicked_cb (GtkWidget * dummy,
     g_free (filename);
     return;
   }
-  ue = GNUNET_malloc (sizeof (struct UnindexEntry));
+  ue = GNUNET_new (struct UnindexEntry);
   ue->filesize = filesize;
   GNUNET_CONTAINER_DLL_insert (ue_head, ue_tail, ue);
   ue->filename = GNUNET_strdup (filename);
@@ -279,8 +279,8 @@ GNUNET_FS_GTK_unindex_close_button_clicked_cb (GtkWidget * dummy,
  * @param ts the changed selection
  * @param user_data our unindex context
  */
-static void
-selection_changed_cb (GtkTreeSelection * ts,
+void
+GNUNET_FS_GTK_unindex_treeview_selection_changed_cb (GtkTreeSelection *ts,
 		      gpointer user_data)
 {
   struct UnindexDialogContext *udc = user_data;
@@ -394,7 +394,7 @@ GNUNET_GTK_main_menu_unindex_activate_cb (GtkWidget * dummy,
     gtk_window_present (GTK_WINDOW (master_udc->dialog));
     return;
   }
-  udc = GNUNET_malloc (sizeof (struct UnindexDialogContext));
+  udc = GNUNET_new (struct UnindexDialogContext);
   udc->builder =
     GNUNET_GTK_get_new_builder ("gnunet_fs_gtk_unindex.glade", udc);
   if (NULL == udc->builder)
@@ -418,9 +418,6 @@ GNUNET_GTK_main_menu_unindex_activate_cb (GtkWidget * dummy,
   toplevel = gtk_widget_get_toplevel (dummy);
   if (GTK_IS_WINDOW (toplevel))
     gtk_window_set_transient_for (GTK_WINDOW (udc->dialog), GTK_WINDOW (toplevel));
-  /* connect signals; FIXME-GTK3: these could be connected with (modern) Glade */
-  g_signal_connect (G_OBJECT (udc->selection), "changed",
-                    G_CALLBACK (selection_changed_cb), udc);
   gtk_window_present (GTK_WINDOW (udc->dialog));
   master_udc = udc;
 }
@@ -446,7 +443,7 @@ GNUNET_FS_GTK_unindex_handle_resume_ (struct GNUNET_FS_UnindexContext *uc,
 {
   struct UnindexEntry *ue;
 
-  ue = GNUNET_malloc (sizeof (struct UnindexEntry));
+  ue = GNUNET_new (struct UnindexEntry);
   ue->filename = GNUNET_strdup (filename);
   if (NULL != emsg)
     ue->emsg = GNUNET_strdup (emsg);
