@@ -8,10 +8,12 @@ gint types_generic[] = {
   EXTRACTOR_METATYPE_SUBJECT,
   EXTRACTOR_METATYPE_DESCRIPTION,
   EXTRACTOR_METATYPE_COMMENT,
+#if HAVE_EXTRACTOR_H
   EXTRACTOR_METATYPE_COPYRIGHT,
   EXTRACTOR_METATYPE_URI,
   EXTRACTOR_METATYPE_CREATOR,
   EXTRACTOR_METATYPE_CREATION_DATE,
+#endif
   EXTRACTOR_METATYPE_RESERVED
 };
 
@@ -22,6 +24,7 @@ gint types_text[] =
   EXTRACTOR_METATYPE_GNUNET_ORIGINAL_FILENAME,
   EXTRACTOR_METATYPE_TITLE,
   EXTRACTOR_METATYPE_KEYWORDS,
+#if HAVE_EXTRACTOR_H
   EXTRACTOR_METATYPE_ABSTRACT,
   EXTRACTOR_METATYPE_SUMMARY,
   EXTRACTOR_METATYPE_SUBJECT,
@@ -58,6 +61,7 @@ gint types_text[] =
   EXTRACTOR_METATYPE_TEMPLATE,
   EXTRACTOR_METATYPE_REVISION_HISTORY,
   EXTRACTOR_METATYPE_REVISION_NUMBER,
+#endif
   EXTRACTOR_METATYPE_RESERVED
 };
 
@@ -68,6 +72,7 @@ gint types_music[] =
   EXTRACTOR_METATYPE_GNUNET_ORIGINAL_FILENAME,
   EXTRACTOR_METATYPE_TITLE,
   EXTRACTOR_METATYPE_ARTIST,
+#if HAVE_EXTRACTOR_H
   EXTRACTOR_METATYPE_COMPOSER,
   EXTRACTOR_METATYPE_CONDUCTOR,
   EXTRACTOR_METATYPE_ALBUM,
@@ -101,6 +106,7 @@ gint types_music[] =
   EXTRACTOR_METATYPE_ORIGINAL_PERFORMER,
   EXTRACTOR_METATYPE_MUSICIAN_CREDITS_LIST,
   EXTRACTOR_METATYPE_SUBTITLE,
+#endif
   EXTRACTOR_METATYPE_RESERVED
 };
 
@@ -110,6 +116,7 @@ gint types_video[] =
   EXTRACTOR_METATYPE_MIMETYPE,
   EXTRACTOR_METATYPE_GNUNET_ORIGINAL_FILENAME,
   EXTRACTOR_METATYPE_TITLE,
+#if HAVE_EXTRACTOR_H
   EXTRACTOR_METATYPE_GENRE,
   EXTRACTOR_METATYPE_KEYWORDS,
   EXTRACTOR_METATYPE_SUMMARY,
@@ -133,6 +140,7 @@ gint types_video[] =
   EXTRACTOR_METATYPE_LOCATION_COUNTRY_CODE,
   EXTRACTOR_METATYPE_CREATION_DATE,
   EXTRACTOR_METATYPE_PERFORMER,
+#endif
   EXTRACTOR_METATYPE_RESERVED
 };
 
@@ -144,6 +152,7 @@ gint types_image[] =
   EXTRACTOR_METATYPE_TITLE,
   EXTRACTOR_METATYPE_KEYWORDS,
   EXTRACTOR_METATYPE_DESCRIPTION,
+#if HAVE_EXTRACTOR_H
   EXTRACTOR_METATYPE_IMAGE_DIMENSIONS,
   EXTRACTOR_METATYPE_IMAGE_RESOLUTION,
   EXTRACTOR_METATYPE_COMMENT,
@@ -156,6 +165,7 @@ gint types_image[] =
   EXTRACTOR_METATYPE_LOCATION_COUNTRY,
   EXTRACTOR_METATYPE_LOCATION_COUNTRY_CODE,
   EXTRACTOR_METATYPE_ORGANIZATION,
+#endif
   EXTRACTOR_METATYPE_RESERVED
 };
 
@@ -167,6 +177,7 @@ gint types_namespace[] =
   EXTRACTOR_METATYPE_COMMENT,
   EXTRACTOR_METATYPE_TITLE,
   EXTRACTOR_METATYPE_DESCRIPTION,
+#if HAVE_EXTRACTOR_H
   EXTRACTOR_METATYPE_COPYRIGHT,
   EXTRACTOR_METATYPE_RIGHTS,
   EXTRACTOR_METATYPE_KEYWORDS,
@@ -179,6 +190,7 @@ gint types_namespace[] =
   EXTRACTOR_METATYPE_RIPPER,
   EXTRACTOR_METATYPE_PRODUCER,
   EXTRACTOR_METATYPE_GROUP,
+#endif
   EXTRACTOR_METATYPE_RESERVED
 };
 
@@ -188,6 +200,7 @@ gint types_software[] =
   EXTRACTOR_METATYPE_MIMETYPE,
   EXTRACTOR_METATYPE_GNUNET_ORIGINAL_FILENAME,
   EXTRACTOR_METATYPE_PACKAGE_NAME,
+#if HAVE_EXTRACTOR_H
   EXTRACTOR_METATYPE_PACKAGE_VERSION,
   EXTRACTOR_METATYPE_TARGET_ARCHITECTURE,
   EXTRACTOR_METATYPE_LICENSE,
@@ -214,6 +227,7 @@ gint types_software[] =
   EXTRACTOR_METATYPE_LIBRARY_SEARCH_PATH,
   EXTRACTOR_METATYPE_LIBRARY_DEPENDENCY,
   EXTRACTOR_METATYPE_UPLOAD_PRIORITY,
+#endif
   EXTRACTOR_METATYPE_RESERVED
 };
 
