@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2012 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2012 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -38,17 +38,17 @@
  * Handle for an animation.  Each animation file should only
  * be loaded once and can then be used in multiple tree views.
  */
-struct GNUNET_FS_AnimationContext
+struct GNUNET_GTK_AnimationContext
 {
   /**
    * This is a doublye-linked list.
    */
-  struct GNUNET_FS_AnimationContext *next;
+  struct GNUNET_GTK_AnimationContext *next;
 
   /**
    * This is a doublye-linked list.
    */
-  struct GNUNET_FS_AnimationContext *prev;
+  struct GNUNET_GTK_AnimationContext *prev;
 
   /**
    * Handle to the animation.
@@ -74,17 +74,17 @@ struct GNUNET_FS_AnimationContext
  * must be told about the tree view and column for the animations
  * to work.
  */
-struct GNUNET_FS_AnimationTreeViewHandle
+struct GNUNET_GTK_AnimationTreeViewHandle
 {
   /**
    * This is a doublye-linked list.
    */
-  struct GNUNET_FS_AnimationTreeViewHandle *next;
+  struct GNUNET_GTK_AnimationTreeViewHandle *next;
 
   /**
    * This is a doublye-linked list.
    */
-  struct GNUNET_FS_AnimationTreeViewHandle *prev;
+  struct GNUNET_GTK_AnimationTreeViewHandle *prev;
 
   /**
    * Tree view to watch.
@@ -101,27 +101,27 @@ struct GNUNET_FS_AnimationTreeViewHandle
 /**
  * Head of linked list of animations.
  */
-static struct GNUNET_FS_AnimationContext *animation_head;
+static struct GNUNET_GTK_AnimationContext *animation_head;
 
 /**
  * Tail of linked list of animations.
  */
-static struct GNUNET_FS_AnimationContext *animation_tail;
+static struct GNUNET_GTK_AnimationContext *animation_tail;
 
 /**
  * Head of linked list of tree views with animations.
  */
-static struct GNUNET_FS_AnimationTreeViewHandle *atv_head;
+static struct GNUNET_GTK_AnimationTreeViewHandle *atv_head;
 
 /**
  * Tail of linked list of tree views with animations.
  */
-static struct GNUNET_FS_AnimationTreeViewHandle *atv_tail;
+static struct GNUNET_GTK_AnimationTreeViewHandle *atv_tail;
 
 /**
  * Task run to update animations.
  */
-static GNUNET_SCHEDULER_TaskIdentifier ticker_task;
+static struct GNUNET_SCHEDULER_Task *ticker_task;
 
 
 /**
@@ -132,13 +132,13 @@ static GNUNET_SCHEDULER_TaskIdentifier ticker_task;
  * @param filename name of the file with the animation to show
  * @return context to use to get the animated pixbuf, NULL on error
  */
-struct GNUNET_FS_AnimationContext *
+struct GNUNET_GTK_AnimationContext *
 GNUNET_GTK_animation_context_create (const char *filename)
 {
   GError *err = NULL;
-  struct GNUNET_FS_AnimationContext *ac;
+  struct GNUNET_GTK_AnimationContext *ac;
 
-  ac = GNUNET_malloc (sizeof (struct GNUNET_FS_AnimationContext));
+  ac = GNUNET_new (struct GNUNET_GTK_AnimationContext);
   ac->ani = gdk_pixbuf_animation_new_from_file (filename, &err);
   if (NULL == ac->ani)
     {
@@ -164,7 +164,7 @@ GNUNET_GTK_animation_context_create (const char *filename)
  * @param ac animation context to destroy.
  */
 void
-GNUNET_GTK_animation_context_destroy (struct GNUNET_FS_AnimationContext *ac)
+GNUNET_GTK_animation_context_destroy (struct GNUNET_GTK_AnimationContext *ac)
 {
   if (NULL == ac)
     return;
@@ -188,7 +188,7 @@ GNUNET_GTK_animation_context_destroy (struct GNUNET_FS_AnimationContext *ac)
  * @return pixbuf of the AC, NULL on error loading the pixbuf
  */
 GdkPixbuf *
-GNUNET_GTK_animation_context_get_pixbuf (struct GNUNET_FS_AnimationContext *ac)
+GNUNET_GTK_animation_context_get_pixbuf (struct GNUNET_GTK_AnimationContext *ac)
 {
   if (NULL == ac)
     return NULL;
@@ -203,7 +203,7 @@ GNUNET_GTK_animation_context_get_pixbuf (struct GNUNET_FS_AnimationContext *ac)
  * @return 0 if nothing needed to be done
  */
 static int
-tick_animation (struct GNUNET_FS_AnimationContext *ac)
+tick_animation (struct GNUNET_GTK_AnimationContext *ac)
 {
   GdkPixbuf *pixbuf;
   gint width;
@@ -226,7 +226,7 @@ tick_animation (struct GNUNET_FS_AnimationContext *ac)
  * @param atv handle to the tree view to redraw
  */
 static void
-redraw_tree_view (struct GNUNET_FS_AnimationTreeViewHandle *atv)
+redraw_tree_view (struct GNUNET_GTK_AnimationTreeViewHandle *atv)
 {
   GdkWindow *gw;
   GdkRectangle r;
@@ -261,14 +261,12 @@ redraw_tree_view (struct GNUNET_FS_AnimationTreeViewHandle *atv)
  * Task run periodically to advance all of our animations.
  *
  * @param cls closure, unused
- * @param tc scheduler context, unused
  */
 static void
-ticker (void *cls,
-	const struct GNUNET_SCHEDULER_TaskContext *tc)
+ticker (void *cls)
 {
-  struct GNUNET_FS_AnimationContext *pos;
-  struct GNUNET_FS_AnimationTreeViewHandle *atv;
+  struct GNUNET_GTK_AnimationContext *pos;
+  struct GNUNET_GTK_AnimationTreeViewHandle *atv;
   unsigned int counter;
 
   ticker_task = GNUNET_SCHEDULER_add_delayed (TICKER_DELAY,
@@ -293,19 +291,19 @@ ticker (void *cls,
  * @param image_col column in the tree view with the animated pixbufs
  * @return handle to unregister the tree view later
  */
-struct GNUNET_FS_AnimationTreeViewHandle *
+struct GNUNET_GTK_AnimationTreeViewHandle *
 GNUNET_GTK_animation_tree_view_register (GtkTreeView *tv,
 					 GtkTreeViewColumn *image_col)
 {
-  struct GNUNET_FS_AnimationTreeViewHandle *atv;
+  struct GNUNET_GTK_AnimationTreeViewHandle *atv;
 
-  atv = GNUNET_malloc (sizeof (struct GNUNET_FS_AnimationTreeViewHandle));
+  atv = GNUNET_new (struct GNUNET_GTK_AnimationTreeViewHandle);
   atv->tv = tv;
   atv->image_col = image_col;
   GNUNET_CONTAINER_DLL_insert (atv_head,
 			       atv_tail,
 			       atv);
-  if (GNUNET_SCHEDULER_NO_TASK == ticker_task)
+  if (NULL == ticker_task)
     ticker_task = GNUNET_SCHEDULER_add_delayed (TICKER_DELAY,
 						&ticker,
 						NULL);
@@ -319,7 +317,7 @@ GNUNET_GTK_animation_tree_view_register (GtkTreeView *tv,
  * @param atv tree view to unregister
  */
 void
-GNUNET_GTK_animation_tree_view_unregister (struct GNUNET_FS_AnimationTreeViewHandle *atv)
+GNUNET_GTK_animation_tree_view_unregister (struct GNUNET_GTK_AnimationTreeViewHandle *atv)
 {
   GNUNET_CONTAINER_DLL_remove (atv_head,
 			       atv_tail,
@@ -328,7 +326,7 @@ GNUNET_GTK_animation_tree_view_unregister (struct GNUNET_FS_AnimationTreeViewHan
   if (NULL != atv_head)
     return;
   GNUNET_SCHEDULER_cancel (ticker_task);
-  ticker_task = GNUNET_SCHEDULER_NO_TASK;
+  ticker_task = NULL;
 }
 
 
