@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2010-2014 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2010-2014 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -35,6 +35,14 @@
 #include <unique/unique.h>
 #endif
 
+#define UPDATE_FREQUENCY GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_SECONDS, 1)
+
+/**
+ * Should we show peers that have no connections and
+ * no known (valid, non-expired) addresses?
+ */
+static int show_inactive;
+
 
 /**
  * Columns in the peerinfo model.
@@ -82,7 +90,7 @@ enum PEERINFO_ModelColumns
   PEERINFO_MC_CORE_CONNECTIVITY_LED = 7,
 
   /**
-   * A gboolean
+   * A guint for sorting by connectivity
    */
   PEERINFO_MC_CORE_CONNECTED_STATUS = 8,
 
@@ -134,42 +142,32 @@ enum PEERINFO_ModelColumns
   /**
    * A GdkPixbuf
    */
-  PEERINFO_MC_VALIDATION_STATE_LED = 18,
+  PEERINFO_MC_PLUGIN_CONNECTIVITY_LED = 18,
 
   /**
    * A gboolean
    */
-  PEERINFO_MC_VALIDATION_IS_VALID = 19,
+  PEERINFO_MC_PLUGIN_CONNECTIVITY_STATUS = 19,
 
   /**
    * A gchararray
    */
-  PEERINFO_MC_VALIDATION_TIMEOUT_AS_STRING = 20,
-
-  /**
-   * A GdkPixbuf
-   */
-  PEERINFO_MC_PLUGIN_CONNECTIVITY_LED = 21,
+  PEERINFO_MC_PLUGIN_CONNECTIVITY_TIMEOUT_AS_STRING = 20,
 
   /**
    * A gboolean
    */
-  PEERINFO_MC_PLUGIN_CONNECTIVITY_STATUS = 22,
+  PEERINFO_MC_SHOW_FRIEND = 21,
 
   /**
    * A gchararray
    */
-  PEERINFO_MC_PLUGIN_CONNECTIVITY_TIMEOUT_AS_STRING = 23,
+  PEERINFO_MC_PEERINFO_ADDRESS_EXPIRATION = 22,
 
   /**
-   * A gboolean
+   * Last "column" (does not actually exist).
    */
-  PEERINFO_MC_SHOW_FRIEND = 24,
-
-  /**
-   * A gchararray
-   */
-  PEERINFO_MC_PEERINFO_ADDRESS_EXPIRATION = 25
+  PEERINFO_MC_END = 23
 
 };
 
@@ -312,7 +310,7 @@ static struct GNUNET_PEERINFO_NotifyContext *pnc;
 static struct GNUNET_ATS_PerformanceHandle *ats;
 
 /**
- * Map of peer identities to the respective PeerInfo for our view.
+ * Map of peer identities to the respective `struct PeerInfo` for our view.
  */
 static struct GNUNET_CONTAINER_MultiPeerMap *peer2info;
 
@@ -322,19 +320,14 @@ static struct GNUNET_CONTAINER_MultiPeerMap *peer2info;
 static struct GNUNET_TRANSPORT_PeerMonitoringContext *pmc;
 
 /**
- * Monitoring transport validation operations.
+ * Monitoring transport plugin sessions.
  */
-static struct GNUNET_TRANSPORT_ValidationMonitoringContext *vmc;
+struct GNUNET_TRANSPORT_PluginMonitor *pm;
 
 /**
  * Monitoring core connectivity.
  */
-static struct GNUNET_CORE_Handle *core;
-
-/**
- * Should gnunet-peerinfo-gtk start in tray mode?
- */
-static int tray_only;
+static struct GNUNET_CORE_MonitorHandle *cm;
 
 /**
  * Green status led (connected)
@@ -346,6 +339,21 @@ static GdkPixbuf *led_green;
  */
 static GdkPixbuf *led_red;
 
+/**
+ * Black status led (error)
+ */
+static GdkPixbuf *led_black;
+
+/**
+ * Yellow status led (connecting)
+ */
+static GdkPixbuf *led_yellow;
+
+/**
+ * Blue status led (reconnecting/rekeying/special operation)
+ */
+static GdkPixbuf *led_blue;
+
 /**
  * Main window tree store.
  */
@@ -377,14 +385,14 @@ static GtkTreeViewColumn *tvc_ats_connectivity;
 static GtkTreeViewColumn *tvc_plugin_connectivity;
 
 /**
- * Tree view column for validation status
+ * The main tree view.
  */
-static GtkTreeViewColumn *tvc_validation_status;
+static GtkTreeView *tv;
 
 /**
- * Tree view column for neighbour_state
+ * Task to actually update the tree view.
  */
-static GtkTreeViewColumn *tvc_neighbour_state;
+static struct GNUNET_SCHEDULER_Task *update_task;
 
 
 #if HAVE_LIBUNIQUE
@@ -392,6 +400,36 @@ static UniqueApp *unique_app;
 #endif
 
 
+/**
+ * Periodically unfreezes the tree view to allow updates to show.
+ *
+ * @param cls NULL
+ */
+static void
+do_update (void *cls)
+{
+  static int phase;
+
+  (void) cls;
+  phase++;
+  if (0 == (phase % 2))
+  {
+    gtk_widget_thaw_child_notify (GTK_WIDGET (tv));
+    update_task = GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_MILLISECONDS,
+						&do_update,
+						NULL);
+
+  }
+  else
+  {
+    gtk_widget_freeze_child_notify (GTK_WIDGET (tv));
+    update_task = GNUNET_SCHEDULER_add_delayed (UPDATE_FREQUENCY,
+						&do_update,
+						NULL);
+  }
+}
+
+
 /**
  * Get cfg.
  */
@@ -415,6 +453,33 @@ get_object (const char *name)
 }
 
 
+/**
+ * Free the given peer address which must belong to the
+ * given peer.
+ *
+ * @param info peer the address belongs to
+ * @param pa address to free
+ */
+static void
+free_peer_address (struct PeerInfo *info,
+                   struct PeerAddress *pa)
+{
+  GNUNET_CONTAINER_DLL_remove (info->pa_head,
+                               info->pa_tail,
+                               pa);
+  if (NULL != pa->tos)
+  {
+    GNUNET_TRANSPORT_address_to_string_cancel (pa->tos);
+    pa->tos = NULL;
+  }
+  gtk_tree_row_reference_free (pa->rr);
+  GNUNET_free_non_null (pa->plugin);
+  GNUNET_free_non_null (pa->address_as_string);
+  GNUNET_free_non_null (pa->country_name);
+  GNUNET_free (pa);
+}
+
+
 /**
  * Function called on each entry in the #peer2info map
  * to free the associated path.
@@ -433,21 +498,8 @@ free_paths (void *cts,
   struct PeerAddress *pa;
 
   while (NULL != (pa = info->pa_head))
-  {
-    GNUNET_CONTAINER_DLL_remove (info->pa_head,
-				 info->pa_tail,
-				 pa);
-    if (NULL != pa->tos)
-    {
-      GNUNET_TRANSPORT_address_to_string_cancel (pa->tos);
-      pa->tos = NULL;
-    }
-    gtk_tree_row_reference_free (pa->rr);
-    GNUNET_free_non_null (pa->plugin);
-    GNUNET_free_non_null (pa->address_as_string);
-    GNUNET_free_non_null (pa->country_name);
-    GNUNET_free (pa);
-  }
+    free_peer_address (info,
+                       pa);
   gtk_tree_row_reference_free (info->rr);
   GNUNET_free (info);
   return GNUNET_OK;
@@ -458,15 +510,21 @@ free_paths (void *cts,
  * Task run on shutdown.
  *
  * @param cts unused
- * @param tc scheduler context, unused
  */
 static void
-shutdown_task (void *cts,
-	       const struct GNUNET_SCHEDULER_TaskContext *tc)
+shutdown_task (void *cts)
 {
-  GNUNET_GTK_tray_icon_destroy ();
+  if (NULL != update_task)
+  {
+    GNUNET_SCHEDULER_cancel (update_task);
+    update_task = NULL;
+  }
+  if (NULL == ml)
+  {
+    GNUNET_break (0);
+    return;
+  }
   GNUNET_GTK_main_loop_quit (ml);
-  ml = NULL;
   if (NULL != pnc)
   {
     GNUNET_PEERINFO_notify_cancel (pnc);
@@ -482,17 +540,19 @@ shutdown_task (void *cts,
     GNUNET_TRANSPORT_monitor_peers_cancel (pmc);
     pmc = NULL;
   }
-  if (NULL != vmc)
+  if (NULL != pm )
   {
-    GNUNET_TRANSPORT_monitor_validation_entries_cancel (vmc);
-    vmc = NULL;
+    GNUNET_TRANSPORT_monitor_plugins_cancel (pm);
+    pm = NULL;
   }
-  if (NULL != core)
+  if (NULL != cm)
   {
-    GNUNET_CORE_disconnect (core);
-    core = NULL;
+    GNUNET_CORE_monitor_stop (cm);
+    cm = NULL;
   }
-  GNUNET_CONTAINER_multipeermap_iterate (peer2info, &free_paths, NULL);
+  GNUNET_CONTAINER_multipeermap_iterate (peer2info,
+                                         &free_paths,
+                                         NULL);
   GNUNET_CONTAINER_multipeermap_destroy (peer2info);
   peer2info = NULL;
   GNUNET_PEERINFO_GTK_flags_shutdown ();
@@ -513,7 +573,9 @@ get_iter_from_rr (GtkTreeRowReference *rr,
 
   path = gtk_tree_row_reference_get_path (rr);
   GNUNET_assert (NULL != path);
-  GNUNET_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (ts), iter, path));
+  GNUNET_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (ts),
+                                          iter,
+                                          path));
   gtk_tree_path_free (path);
 }
 
@@ -522,11 +584,15 @@ get_iter_from_rr (GtkTreeRowReference *rr,
  * Function to call with the text format of an address
  *
  * @param cts the `struct PeerAddress` for the address
- * @param address address as a string, NULL on error
+ * @param address address as a string, NULL on last call
+ * @param res result of the address to string conversion:
+ *        if #GNUNET_OK: address contains address as string
+ *        if #GNUNET_SYSERR: address is invalid
  */
 static void
 peer_address_string_cb (void *cts,
-			const char *address)
+			const char *address,
+                        int res)
 {
   struct PeerAddress *pa = cts;
   GtkTreeIter iter;
@@ -539,6 +605,8 @@ peer_address_string_cb (void *cts,
     pa->tos = NULL;
     return;
   }
+  if (GNUNET_SYSERR == res)
+    return; /* don't care */
   get_iter_from_rr (pa->rr, &iter);
   country = NULL;
   colon = strstr (address, ":");
@@ -607,20 +675,28 @@ get_address (struct PeerInfo *pi,
                                pa);
   path = gtk_tree_row_reference_get_path (pi->rr);
   GNUNET_assert (NULL != path);
-  GNUNET_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (ts), &iter, path));
+  GNUNET_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (ts),
+					  &iter,
+					  path));
   gtk_tree_path_free (path);
-  gtk_tree_store_append (ts, &aiter, &iter);
-  gtk_tree_store_set (ts, &iter,
-		      PEERINFO_MC_PLUGIN_NAME, addr->transport_name,
-		      -1);
-  path = gtk_tree_model_get_path (GTK_TREE_MODEL (ts), &aiter);
-  pa->rr = gtk_tree_row_reference_new (GTK_TREE_MODEL (ts), path);
+  gtk_tree_store_insert_with_values (ts,
+                                     &aiter,
+                                     &iter,
+                                     -1 /* append */,
+                                     PEERINFO_MC_PLUGIN_NAME, addr->transport_name,
+                                     -1);
+  path = gtk_tree_model_get_path (GTK_TREE_MODEL (ts),
+				  &aiter);
+  pa->rr = gtk_tree_row_reference_new (GTK_TREE_MODEL (ts),
+				       path);
   GNUNET_assert (NULL != pa->rr);
   gtk_tree_path_free (path);
-  pa->tos = GNUNET_TRANSPORT_address_to_string (get_configuration (), addr,
+  pa->tos = GNUNET_TRANSPORT_address_to_string (get_configuration (),
+						addr,
                                                 GNUNET_NO,
                                                 GNUNET_TIME_UNIT_FOREVER_REL,
-                                                &peer_address_string_cb, pa);
+                                                &peer_address_string_cb,
+						pa);
 
   return pa;
 }
@@ -646,7 +722,9 @@ peer_address_cb (void *cts,
 
   path = gtk_tree_row_reference_get_path (info->rr);
   GNUNET_assert (NULL != path);
-  GNUNET_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (ts), &iter, path));
+  GNUNET_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (ts),
+					  &iter,
+					  path));
   gtk_tree_path_free (path);
   pa = get_address (info, address);
   get_iter_from_rr (pa->rr, &iter);
@@ -690,14 +768,17 @@ get_peer_info (const struct GNUNET_PeerIdentity *peer)
                       PEERINFO_MC_PEER_IDENTITY_STRING, GNUNET_i2s (peer),
                       PEERINFO_MC_PEERINFO, info,
                       PEERINFO_MC_SHOW_FRIEND, TRUE,
-                      PEERINFO_MC_CORE_CONNECTIVITY_LED, led_red,
                       -1);
-  path = gtk_tree_model_get_path (GTK_TREE_MODEL (ts), &iter);
-  info->rr = gtk_tree_row_reference_new (GTK_TREE_MODEL (ts), path);
+  path = gtk_tree_model_get_path (GTK_TREE_MODEL (ts),
+				  &iter);
+  info->rr = gtk_tree_row_reference_new (GTK_TREE_MODEL (ts),
+					 path);
   GNUNET_assert (NULL != info->rr);
   gtk_tree_path_free (path);
   GNUNET_assert (GNUNET_OK ==
-                 GNUNET_CONTAINER_multipeermap_put (peer2info, peer, info,
+                 GNUNET_CONTAINER_multipeermap_put (peer2info,
+						    peer,
+						    info,
                                                     GNUNET_CONTAINER_MULTIHASHMAPOPTION_UNIQUE_ONLY));
   return info;
 }
@@ -719,62 +800,122 @@ peerinfo_processor (void *cts,
 {
   struct PeerInfo *info;
 
-  info = get_peer_info (peer);
-  if (NULL == hello)
+  if ( (NULL == hello) ||
+       ( (! show_inactive) &&
+         (0 ==
+          GNUNET_TIME_absolute_get_remaining (GNUNET_HELLO_get_last_expiration (hello)).rel_value_us) ) )
     return;
+  info = get_peer_info (peer);
   GNUNET_HELLO_iterate_addresses (hello, GNUNET_NO,
                                   &peer_address_cb, info);
 }
 
 
+/**
+ * ATS disconnected, remove ATS status information for all peers.
+ *
+ * @param cls NULL
+ * @param pid peer identity to reset
+ * @param value the `struct PeerInfo` for the peer
+ * @return #GNUNET_OK
+ */
+static int
+reset_ats_status (void *cls,
+                   const struct GNUNET_PeerIdentity *pid,
+                   void *value)
+{
+  struct PeerInfo *info = value;
+  GtkTreeIter iter;
+
+  get_iter_from_rr (info->rr, &iter);
+  gtk_tree_store_set (ts, &iter,
+                      PEERINFO_MC_ATS_CONNECTIVITY_LED, NULL,
+                      PEERINFO_MC_ATS_SELECTED_STATUS, FALSE,
+		      PEERINFO_MC_BANDWIDTH_IN, 0,
+		      PEERINFO_MC_BANDWIDTH_OUT, 0,
+                      -1);
+  return GNUNET_OK;
+}
+
+
 /**
  * Method called whenever a given peer has a QoS status change.
  *
  * @param cts closure
- * @param address the address
- * @param address_active is this address actively used to maintain a connection
- * 				to a peer
+ * @param address the address, or NULL if service disconnected
+ * @param address_active #GNUNET_YES if this address is actively used
+ *        to maintain a connection to a peer;
+ *        #GNUNET_NO if the address is not actively used;
+ *        #GNUNET_SYSERR if this address is no longer available for ATS
  * @param bandwidth_in available amount of inbound bandwidth
  * @param bandwidth_out available amount of outbound bandwidth
- * @param ats performance data for the address (as far as known)
- * @param ats_count number of performance records in @a ats
+ * @param perf performance data for the address (as far as known)
  */
 static void
-status_cb (void *cts,
-	   const struct GNUNET_HELLO_Address *address,
-	   int address_active,
-           struct GNUNET_BANDWIDTH_Value32NBO bandwidth_in,
-           struct GNUNET_BANDWIDTH_Value32NBO bandwidth_out,
-           const struct GNUNET_ATS_Information *ats,
-	   uint32_t ats_count)
+ats_status_cb (void *cts,
+               const struct GNUNET_HELLO_Address *address,
+               int address_active,
+               struct GNUNET_BANDWIDTH_Value32NBO bandwidth_in,
+               struct GNUNET_BANDWIDTH_Value32NBO bandwidth_out,
+               const struct GNUNET_ATS_Properties *prop)
 {
   struct PeerInfo *info;
   struct PeerAddress *pa;
   struct PeerAddress *act;
   GtkTreeIter iter;
 
+  if (NULL == address)
+  {
+    /* state 'reset' / disconnect from service, reset everything */
+    GNUNET_CONTAINER_multipeermap_iterate (peer2info,
+                                           &reset_ats_status,
+                                           NULL);
+    return;
+  }
+
   info = get_peer_info (&address->peer);
   pa = get_address (info, address);
   pa->address_ats_active = address_active;
   pa->ats_in = (guint) ntohl (bandwidth_in.value__);
   pa->ats_out = (guint) ntohl (bandwidth_out.value__);
   get_iter_from_rr (pa->rr, &iter);
+  if (GNUNET_SYSERR == address_active)
+  {
+    gtk_tree_store_remove (ts, &iter);
+    free_peer_address (info, pa);
+    return;
+  }
   gtk_tree_store_set (ts, &iter,
                       PEERINFO_MC_ATS_CONNECTIVITY_LED, (pa->address_ats_active) ? led_green : led_red,
                       PEERINFO_MC_ATS_SELECTED_STATUS, pa->address_ats_active,
 		      PEERINFO_MC_BANDWIDTH_IN, pa->ats_in,
 		      PEERINFO_MC_BANDWIDTH_OUT, pa->ats_out,
 		      -1);
-  act = NULL;
+  act = (GNUNET_YES == address_active) ? pa : NULL;
   for (pa = info->pa_head; NULL != pa; pa = pa->next)
-    if (pa->address_ats_active)
+  {
+    if (GNUNET_YES == pa->address_ats_active)
     {
-      GNUNET_break (NULL == act);
-      act = pa;
+      if (NULL == act)
+      {
+        /* first active address found, remember it! */
+        act = pa;
+      }
+      else if (GNUNET_YES == address_active)
+      {
+        /* second address became active, set BW for first to zero */
+        get_iter_from_rr (pa->rr, &iter);
+        gtk_tree_store_set (ts, &iter,
+                            PEERINFO_MC_BANDWIDTH_IN, (guint) 0,
+                            PEERINFO_MC_BANDWIDTH_OUT, (guint) 0,
+                            -1);
+      }
     }
+  }
   get_iter_from_rr (info->rr, &iter);
   if (NULL == act)
   {
+    /* no active address, make sure per-peer state is on 'off' */
     gtk_tree_store_set (ts, &iter,
                         PEERINFO_MC_ATS_CONNECTIVITY_LED, led_red,
                         PEERINFO_MC_ATS_SELECTED_STATUS, FALSE,
@@ -788,6 +929,7 @@ status_cb (void *cts,
   }
   else
   {
+    /* update per-peer state to that of active address */
     gtk_tree_store_set (ts, &iter,
                         PEERINFO_MC_ATS_CONNECTIVITY_LED, led_green,
                         PEERINFO_MC_ATS_SELECTED_STATUS, TRUE,
@@ -802,6 +944,47 @@ status_cb (void *cts,
 }
 
 
+/**
+ * Transport peer monitor disconnected, reset
+ * transport peer status information for all peers.
+ *
+ * @param cls NULL
+ * @param pid peer identity to reset
+ * @param value the `struct PeerInfo` for the peer
+ * @return #GNUNET_OK
+ */
+static int
+reset_transport_peer_status (void *cls,
+                             const struct GNUNET_PeerIdentity *pid,
+                             void *value)
+{
+  struct PeerInfo *info = value;
+  GtkTreeIter iter;
+  struct PeerAddress *pa;
+
+  get_iter_from_rr (info->rr, &iter);
+  gtk_tree_store_set (ts, &iter,
+                      PEERINFO_MC_NEIGHBOUR_CONNECTED_STATUS, NULL,
+                      PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_TIMEOUT_AS_STRING, NULL,
+                      PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, NULL,
+                      PEERINFO_MC_NEIGHBOUR_STATE_AS_STRING, NULL,
+                      PEERINFO_MC_NEIGHBOUR_STATE_TIMEOUT_AS_STRING, NULL,
+                      -1);
+  for (pa = info->pa_head; NULL != pa; pa = pa->next)
+  {
+    get_iter_from_rr (pa->rr, &iter);
+    gtk_tree_store_set (ts, &iter,
+                        PEERINFO_MC_NEIGHBOUR_CONNECTED_STATUS, FALSE,
+                        PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_TIMEOUT_AS_STRING, NULL,
+                        PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, NULL,
+                        PEERINFO_MC_NEIGHBOUR_STATE_AS_STRING, NULL,
+                        PEERINFO_MC_NEIGHBOUR_STATE_TIMEOUT_AS_STRING, NULL,
+                        -1);
+  }
+  return GNUNET_OK;
+}
+
+
 /**
  * Function to call with information about a peer
  *
@@ -824,7 +1007,56 @@ transport_peer_cb (void *cts,
   gboolean con;
   const char *tos;
   struct PeerAddress *pa;
+  GdkPixbuf *led;
 
+  if (NULL == peer)
+  {
+    /* disconnect notification, reset all state */
+    GNUNET_CONTAINER_multipeermap_iterate (peer2info,
+                                           &reset_transport_peer_status,
+                                           NULL);
+    return;
+  }
+  switch (state)
+  {
+  case GNUNET_TRANSPORT_PS_NOT_CONNECTED:
+    led = led_red;
+    break;
+  case GNUNET_TRANSPORT_PS_INIT_ATS:
+    led = led_yellow;
+    break;
+  case GNUNET_TRANSPORT_PS_SYN_SENT:
+    led = led_yellow;
+    break;
+  case GNUNET_TRANSPORT_PS_SYN_RECV_ATS:
+    led = led_yellow;
+    break;
+  case GNUNET_TRANSPORT_PS_SYN_RECV_ACK:
+    led = led_yellow;
+    break;
+  case GNUNET_TRANSPORT_PS_CONNECTED:
+    led = led_green;
+    break;
+  case GNUNET_TRANSPORT_PS_RECONNECT_ATS:
+    led = led_blue;
+    break;
+  case GNUNET_TRANSPORT_PS_RECONNECT_SENT:
+    led = led_blue;
+    break;
+  case GNUNET_TRANSPORT_PS_SWITCH_SYN_SENT:
+    led = led_green;
+    break;
+  case GNUNET_TRANSPORT_PS_DISCONNECT:
+    led = led_red;
+    break;
+  case GNUNET_TRANSPORT_PS_DISCONNECT_FINISHED:
+    led = NULL;
+    break;
+  default:
+    GNUNET_break (0);
+    led = NULL;
+    break;
+  }
   info = get_peer_info (peer);
   info->state = state;
   info->state_timeout = state_timeout;
@@ -834,7 +1066,7 @@ transport_peer_cb (void *cts,
   gtk_tree_store_set (ts, &iter,
                       PEERINFO_MC_NEIGHBOUR_CONNECTED_STATUS, con,
                       PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_TIMEOUT_AS_STRING, con ? tos : NULL,
-                      PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, (con ? led_green : led_red),
+                      PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, led,
                       PEERINFO_MC_NEIGHBOUR_STATE_AS_STRING, GNUNET_TRANSPORT_ps2s (state),
                       PEERINFO_MC_NEIGHBOUR_STATE_TIMEOUT_AS_STRING, tos,
                       -1);
@@ -844,7 +1076,7 @@ transport_peer_cb (void *cts,
     gtk_tree_store_set (ts, &iter,
                         PEERINFO_MC_NEIGHBOUR_CONNECTED_STATUS, FALSE,
                         PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_TIMEOUT_AS_STRING, NULL,
-                        PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, led_red,
+                        PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, NULL,
                         PEERINFO_MC_NEIGHBOUR_STATE_AS_STRING, NULL,
                         PEERINFO_MC_NEIGHBOUR_STATE_TIMEOUT_AS_STRING, NULL,
                         -1);
@@ -856,7 +1088,7 @@ transport_peer_cb (void *cts,
   gtk_tree_store_set (ts, &iter,
                       PEERINFO_MC_NEIGHBOUR_CONNECTED_STATUS, con,
                       PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_TIMEOUT_AS_STRING, con ? tos : NULL,
-                      PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, (con ? led_green : led_red),
+                      PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_LED, led,
                       PEERINFO_MC_NEIGHBOUR_STATE_AS_STRING, GNUNET_TRANSPORT_ps2s (state),
                       PEERINFO_MC_NEIGHBOUR_STATE_TIMEOUT_AS_STRING, tos,
                       -1);
@@ -864,100 +1096,174 @@ transport_peer_cb (void *cts,
 
 
 /**
- * Function to call with validation information about a peer
+ * Function called by the plugin with information about the
+ * current sessions managed by the plugin (for monitoring).
  *
- * @param cts closure
- * @param peer peer this update is about,
- *      NULL if this is the final last callback for a iteration operation
- * @param address address, NULL for disconnect notification in monitor mode
- * @param last_validation when was this address last validated
- * @param valid_until when does this address expire
- * @param next_validation time of the next validation operation
- * @param state state in the validation state machine (FIXME: not used yet)
+ * @param cls closure
+ * @param session session handle this information is about,
+ *        NULL to indicate that we are "in sync" (initial
+ *        iteration complete)
+ * @param session_ctx storage location where the application
+ *        can store data; will point to NULL on #GNUNET_TRANSPORT_SS_INIT,
+ *        and must be reset to NULL on #GNUNET_TRANSPORT_SS_DONE
+ * @param sinfo information about the state of the session,
+ *        NULL if @a session is also NULL and we are
+ *        merely signalling that the initial iteration is over;
+ *        NULL with @a session being non-NULL if the monitor
+ *        was being cancelled while sessions were active
  */
 static void
-validation_monitor_cb (void *cts,
-		       const struct GNUNET_PeerIdentity *peer,
-		       const struct GNUNET_HELLO_Address *address,
-                       struct GNUNET_TIME_Absolute last_validation,
-		       struct GNUNET_TIME_Absolute valid_until,
-		       struct GNUNET_TIME_Absolute next_validation,
-                       enum GNUNET_TRANSPORT_ValidationState state)
+transport_plugin_cb (void *cls,
+		     struct GNUNET_TRANSPORT_PluginSession *session,
+		     void **session_ctx,
+		     const struct GNUNET_TRANSPORT_SessionInfo *sinfo)
 {
   struct PeerInfo *info;
-  struct PeerAddress *pa;
   GtkTreeIter iter;
   const char *tos;
-  gboolean valid;
+  struct PeerAddress *pa;
+  GdkPixbuf *led;
 
-  GNUNET_assert (NULL != peer);
-  info = get_peer_info (peer);
-  if (NULL == address)
+  if ( (NULL == session) &&
+       (NULL == sinfo) )
   {
-    /* disconnect, mark all as down */
-    for (pa = info->pa_head; NULL != pa; pa = pa->next)
-    {
-      get_iter_from_rr (pa->rr, &iter);
-      gtk_tree_store_set (ts, &iter,
-                          PEERINFO_MC_VALIDATION_IS_VALID, FALSE,
-                          PEERINFO_MC_VALIDATION_TIMEOUT_AS_STRING, NULL,
-                          PEERINFO_MC_VALIDATION_STATE_LED, NULL,
-                          -1);
-    }
+    /* initial iteration completed, ignore */
     return;
   }
-  valid = GNUNET_TIME_absolute_get_remaining (valid_until).rel_value_us > 0;
-  pa = get_address (info, address);
+  if (NULL == sinfo)
+  {
+    /* in shutdown, ignore */
+    return;
+  }
+  switch (sinfo->state)
+  {
+  case GNUNET_TRANSPORT_SS_INIT:
+    led = led_red;
+    break;
+  case GNUNET_TRANSPORT_SS_HANDSHAKE:
+    led = led_yellow;
+    break;
+  case GNUNET_TRANSPORT_SS_UP:
+    led = led_green;
+    break;
+  case GNUNET_TRANSPORT_SS_UPDATE:
+    led = led_green;
+    break;
+  case GNUNET_TRANSPORT_SS_DONE:
+    led = NULL;
+    break;
+  default:
+    GNUNET_break (0);
+    led = NULL;
+    break;
+  }
+  if (NULL == sinfo->address)
+  {
+    GNUNET_break (0); // is this OK?
+    return;
+  }
+  info = get_peer_info (&sinfo->address->peer);
+  tos = GNUNET_STRINGS_absolute_time_to_string (sinfo->session_timeout);
+  if (NULL == led)
+    tos = NULL;
+  pa = get_address (info, sinfo->address);
   get_iter_from_rr (pa->rr, &iter);
-  tos = GNUNET_STRINGS_absolute_time_to_string (valid_until);
   gtk_tree_store_set (ts, &iter,
-                      PEERINFO_MC_VALIDATION_IS_VALID, valid,
-                      PEERINFO_MC_VALIDATION_TIMEOUT_AS_STRING, tos,
-                      PEERINFO_MC_VALIDATION_STATE_LED, (valid ? led_green : led_red),
+                      PEERINFO_MC_PLUGIN_CONNECTIVITY_STATUS, (led_green == led),
+                      PEERINFO_MC_PLUGIN_CONNECTIVITY_LED, led,
+                      PEERINFO_MC_PLUGIN_CONNECTIVITY_TIMEOUT_AS_STRING, tos,
                       -1);
 }
 
 
 /**
- * Method called whenever a given peer connects.
+ * Core disconnected, remove CORE KX status information for all peers.
  *
- * @param cts closure
- * @param peer peer identity this notification is about
+ * @param cls NULL
+ * @param pid peer identity to reset
+ * @param value the `struct PeerInfo` for the peer
+ * @return #GNUNET_OK
  */
-static void
-handle_core_connect (void *cts,
-		     const struct GNUNET_PeerIdentity *peer)
+static int
+reset_core_status (void *cls,
+                   const struct GNUNET_PeerIdentity *pid,
+                   void *value)
 {
-  struct PeerInfo *info;
+  struct PeerInfo *info = value;
   GtkTreeIter iter;
 
-  info = get_peer_info (peer);
   get_iter_from_rr (info->rr, &iter);
   gtk_tree_store_set (ts, &iter,
-                      PEERINFO_MC_CORE_CONNECTIVITY_LED, led_green,
-                      PEERINFO_MC_CORE_CONNECTED_STATUS, TRUE,
+                      PEERINFO_MC_CORE_CONNECTIVITY_LED, NULL,
+                      PEERINFO_MC_CORE_CONNECTED_STATUS, (guint) 0,
                       -1);
+  return GNUNET_OK;
 }
 
 
 /**
- * Method called whenever a peer disconnects.
+ * Method called whenever CORE KX state changes for a peer.
  *
  * @param cts closure
  * @param peer peer identity this notification is about
+ * @param state current key exchange state of the peer
+ * @param timeout when does the current state expire
  */
 static void
-handle_core_disconnect (void *cts,
-			const struct GNUNET_PeerIdentity *peer)
+handle_core_monitor_event (void *cts,
+                           const struct GNUNET_PeerIdentity *peer,
+                           enum GNUNET_CORE_KxState state,
+                           struct GNUNET_TIME_Absolute timeout)
 {
   struct PeerInfo *info;
   GtkTreeIter iter;
+  GdkPixbuf *led;
+  guint status;
 
+  switch (state)
+  {
+  case GNUNET_CORE_KX_STATE_DOWN:
+    led = led_black;
+    status = 1;
+    break;
+  case GNUNET_CORE_KX_STATE_KEY_SENT:
+    led = led_yellow;
+    status = 2;
+    break;
+  case GNUNET_CORE_KX_STATE_KEY_RECEIVED:
+    led = led_yellow;
+    status = 2;
+    break;
+  case GNUNET_CORE_KX_STATE_UP:
+    led = led_green;
+    status = 4;
+    break;
+  case GNUNET_CORE_KX_STATE_REKEY_SENT:
+    led = led_blue;
+    status = 4;
+    break;
+  case GNUNET_CORE_KX_PEER_DISCONNECT:
+    led = NULL;
+    status = 1;
+    break;
+  case GNUNET_CORE_KX_ITERATION_FINISHED:
+    return;
+  case GNUNET_CORE_KX_CORE_DISCONNECT:
+    GNUNET_CONTAINER_multipeermap_iterate (peer2info,
+                                           &reset_core_status,
+                                           NULL);
+    return;
+  default:
+    GNUNET_break (0);
+    led = NULL;
+    status = -1;
+    break;
+  }
   info = get_peer_info (peer);
   get_iter_from_rr (info->rr, &iter);
   gtk_tree_store_set (ts, &iter,
-                      PEERINFO_MC_CORE_CONNECTIVITY_LED, led_red,
-                      PEERINFO_MC_CORE_CONNECTED_STATUS, FALSE,
+                      PEERINFO_MC_CORE_CONNECTIVITY_LED, led,
+                      PEERINFO_MC_CORE_CONNECTED_STATUS, status,
                       -1);
 }
 
@@ -1029,16 +1335,20 @@ GNUNET_PEERINFO_GTK_main_window_friends_cellrenderertoggle_toggled_cb (GtkCellRe
     GNUNET_break (0);
     return;
   }
-  if (! gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (ts), &old, path))
+  if (! gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (ts),
+					     &old,
+					     path))
   {
     GNUNET_break (0);
     return;
   }
-  gtk_tree_model_get (GTK_TREE_MODEL (ts), &old,
+  gtk_tree_model_get (GTK_TREE_MODEL (ts),
+		      &old,
                       PEERINFO_MC_PEERINFO, &info,
                       PEERINFO_MC_IS_FRIEND, &oldvalue,
                       -1);
-  gtk_tree_store_set (ts, &old,
+  gtk_tree_store_set (ts,
+		      &old,
                       PEERINFO_MC_IS_FRIEND, ! oldvalue,
                       -1);
   if (oldvalue)
@@ -1067,7 +1377,8 @@ GNUNET_PEERINFO_GTK_main_window_friends_cellrenderertoggle_toggled_cb (GtkCellRe
  * @param user_data unused
  */
 void
-GNUNET_PEERINFO_GTK_quit_cb (GObject * object, gpointer user_data)
+GNUNET_PEERINFO_GTK_quit_cb (GObject * object,
+                             gpointer user_data)
 {
   GNUNET_SCHEDULER_shutdown ();
 }
@@ -1086,7 +1397,7 @@ load_led (const char *color)
   char *dir;
   char *fn;
 
-  dir = GNUNET_GTK_installation_get_path (GNUNET_OS_IPK_DATADIR);
+  dir = GNUNET_OS_installation_get_path (GNUNET_OS_IPK_DATADIR);
   GNUNET_asprintf (&fn, "%s%s.png", dir,
                    color);
   GNUNET_free (dir);
@@ -1161,10 +1472,6 @@ query_tooltip_cb (GtkWidget *widget,
     model_column = PEERINFO_MC_PEERINFO_ADDRESS_EXPIRATION;
   else if (tvc_transport_connectivity == column)
     model_column = PEERINFO_MC_NEIGHBOUR_CONNECTIVITY_TIMEOUT_AS_STRING;
-  else if (tvc_validation_status == column)
-    model_column = PEERINFO_MC_VALIDATION_TIMEOUT_AS_STRING;
-  else if (tvc_neighbour_state == column)
-    model_column = PEERINFO_MC_NEIGHBOUR_STATE_TIMEOUT_AS_STRING;
   if (-1 == model_column)
     return FALSE;
 
@@ -1191,25 +1498,26 @@ query_tooltip_cb (GtkWidget *widget,
  * Actual main function run right after GNUnet's scheduler
  * is initialized.  Initializes up GTK and Glade.
  *
- * @param cts NULL
- * @param tc schedule context
+ * @param cts the main loop context
  */
 static void
-run (void *cts,
-     const struct GNUNET_SCHEDULER_TaskContext *tc)
+run (void *cts)
 {
   GtkWidget *main_window;
-  GtkTreeView *tv;
   const struct GNUNET_CONFIGURATION_Handle *cfg;
   int can_edit_friends;
 
   ml = cts;
   if (GNUNET_OK !=
-      GNUNET_GTK_main_loop_build_window (ml, NULL))
+      GNUNET_GTK_main_loop_build_window (ml,
+                                         NULL))
     return;
   cfg = get_configuration ();
   led_green = load_led ("green");
   led_red = load_led ("red");
+  led_black = load_led ("black");
+  led_yellow = load_led ("yellow");
+  led_blue = load_led ("blue");
   GNUNET_GTK_set_icon_search_path ();
   GNUNET_GTK_setup_nls ();
   can_edit_friends =
@@ -1239,37 +1547,26 @@ run (void *cts,
        FALSE);
   }
   peer2info = GNUNET_CONTAINER_multipeermap_create (256, GNUNET_NO);
-  core = GNUNET_CORE_connect (cfg,
-			      NULL,
-			      NULL,
-			      &handle_core_connect,
-			      &handle_core_disconnect,
-			      NULL, GNUNET_NO,
-			      NULL, GNUNET_NO,
-			      NULL);
+  cm = GNUNET_CORE_monitor_start (cfg,
+                                  &handle_core_monitor_event,
+                                  NULL);
   pnc = GNUNET_PEERINFO_notify (cfg,
 				GNUNET_NO,
 				&peerinfo_processor, NULL);
   pmc = GNUNET_TRANSPORT_monitor_peers (cfg,
 					NULL,
 					GNUNET_NO,
-					GNUNET_TIME_UNIT_FOREVER_REL,
 					&transport_peer_cb,
 					NULL);
-  vmc = GNUNET_TRANSPORT_monitor_validation_entries (cfg,
-						     NULL,
-						     GNUNET_NO,
-						     GNUNET_TIME_UNIT_FOREVER_REL,
-						     &validation_monitor_cb,
-						     NULL);
+  pm = GNUNET_TRANSPORT_monitor_plugins (cfg,
+					 &transport_plugin_cb,
+					 NULL);
   ats = GNUNET_ATS_performance_init (cfg,
-                                     &status_cb, NULL);
+                                     &ats_status_cb, NULL);
   tvc_core_connectivity = GTK_TREE_VIEW_COLUMN (get_object ("GNUNET_PEERINFO_GTK_main_window_core_connectivity_treeviewcolumn"));
   tvc_ats_connectivity = GTK_TREE_VIEW_COLUMN (get_object ("GNUNET_PEERINFO_GTK_main_window_ats_connectivity_treeviewcolumn"));
   tvc_plugin_connectivity = GTK_TREE_VIEW_COLUMN (get_object ("GNUNET_PEERINFO_GTK_main_window_plugin_connectivity_treeviewcolumn"));
   tvc_transport_connectivity = GTK_TREE_VIEW_COLUMN (get_object ("GNUNET_PEERINFO_GTK_main_window_transport_connectivity_treeviewcolumn"));
-  tvc_validation_status = GTK_TREE_VIEW_COLUMN (get_object ("GNUNET_PEERINFO_GTK_main_window_validation_status_treeviewcolumn"));
-  tvc_neighbour_state = GTK_TREE_VIEW_COLUMN (get_object ("GNUNET_PEERINFO_GTK_main_window_neighbour_state_treeviewcolumn"));
   /* setup main window */
   main_window = GTK_WIDGET (get_object ("GNUNET_PEERINFO_GTK_main_window"));
   main_window = GNUNET_GTK_plug_me ("GNUNET_PEERINFO_GTK_PLUG",
@@ -1277,29 +1574,23 @@ run (void *cts,
   ts = GTK_TREE_STORE (get_object ("GNUNET_PEERINFO_GTK_tree_store"));
   GNUNET_assert (NULL != ts);
   gtk_window_maximize (GTK_WINDOW (main_window));
-  if (NULL == getenv ("GNUNET_PEERINFO_GTK_PLUG"))
-    GNUNET_GTK_tray_icon_create (ml,
-                                 GTK_WINDOW (main_window),
-                                 "gnunet-gtk" /* FIXME: different icon? */ ,
-                                 "gnunet-peerinfo-gtk");
-
   tv = GTK_TREE_VIEW (get_object ("GNUNET_PEERINFO_GTK_main_window_treeview"));
   g_object_set (tv, "has-tooltip", TRUE, NULL);
   g_signal_connect (tv, "query-tooltip",
 		    G_CALLBACK (query_tooltip_cb),
                     NULL);
-
 #if HAVE_LIBUNIQUE
-  unique_app_watch_window (unique_app, GTK_WINDOW (main_window));
+  unique_app_watch_window (unique_app,
+                           GTK_WINDOW (main_window));
 #endif
   /* make GUI visible */
-  if (!tray_only)
-  {
-    gtk_widget_show (main_window);
-    gtk_window_present (GTK_WINDOW (main_window));
-  }
-  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
-				&shutdown_task, NULL);
+  gtk_widget_show (main_window);
+  gtk_window_present (GTK_WINDOW (main_window));
+  update_task = GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_SECONDS,
+                                              &do_update,
+                                              NULL);
+  GNUNET_SCHEDULER_add_shutdown (&shutdown_task,
+				 NULL);
 }
 
 
@@ -1313,21 +1604,25 @@ run (void *cts,
 int
 main (int argc, char **argv)
 {
-  static struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'t', "tray", NULL,
-     gettext_noop ("start in tray mode"), 0,
-     &GNUNET_GETOPT_set_one, &tray_only},
+  struct GNUNET_GETOPT_CommandLineOption options[] = {
+    GNUNET_GETOPT_option_flag ('s',
+                               "show-inactive",
+                               gettext_noop ("show peers even if the are inactive and we know nothing except their public key"),
+                               &show_inactive),
     GNUNET_GETOPT_OPTION_END
   };
 
 #if HAVE_LIBUNIQUE
   gtk_init (&argc, &argv);
-  unique_app = unique_app_new ("org.gnunet.gnunet-peerinfo-gtk", NULL);
+  unique_app = unique_app_new ("org.gnunet.gnunet-peerinfo-gtk",
+                               NULL);
   if (unique_app_is_running (unique_app))
   {
     UniqueResponse response;
 
-    response = unique_app_send_message (unique_app, UNIQUE_ACTIVATE, NULL);
+    response = unique_app_send_message (unique_app,
+                                        UNIQUE_ACTIVATE,
+                                        NULL);
     g_object_unref (unique_app);
     return (UNIQUE_RESPONSE_OK == response) ? 0 : 1;
   }
@@ -1335,8 +1630,10 @@ main (int argc, char **argv)
 
   if (GNUNET_OK !=
       GNUNET_GTK_main_loop_start ("gnunet-peerinfo-gtk",
-                                  "GTK GUI for inspecting GNUnet Peers", argc,
-                                  argv, options,
+                                  "GTK GUI for inspecting GNUnet Peers",
+                                  argc,
+                                  argv,
+                                  options,
                                   "gnunet_peerinfo_gtk_main_window.glade",
                                   &run))
     return 1;
