@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2013 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2013 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -82,7 +82,7 @@ add_name_entry_to_list_store (void *cls,
                                        SERVICE_MC_SERVICE_NAME, "",
                                        SERVICE_MC_ISUDP, FALSE,
                                        SERVICE_MC_VISIBLE_PORT, (guint) 80,
-                                       SERVICE_MC_DESTINATION_ADDRESS, "127.0.0.1:8080",
+                                       SERVICE_MC_DESTINATION_ADDRESS, "169.254.86.1:8080",
                                        -1);
     return;
   }
