@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2010, 2012 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2010, 2012 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -31,7 +31,7 @@
  * Test if the configuration works for the given plugin.
  *
  * @param name name of the plugin to check
- * @return GNUNET_OK on success, GNUNET_SYSERR if the config is wrong
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR if the config is wrong
  */
 static int
 test_config (const char *name)
@@ -85,28 +85,4 @@ GNUNET_setup_namestore_postgres_tab_test_button_clicked_cb (GtkWidget * widget,
 }
 
 
-static void
-restart_namestore ()
-{
-  GNUNET_SCHEDULER_add_now (&GNUNET_SETUP_restart_namestore,
-			    NULL);
-}
-
-
-void
-GNUNET_setup_namestore_sqlite_radiobutton_toggled_cb (GtkToggleButton *tb,
-						      gpointer user_data)
-{
-  restart_namestore ();
-}
-
-
-void
-GNUNET_setup_namestore_postgres_radiobutton_toggled_cb (GtkToggleButton *tb,
-							gpointer user_data)
-{
-  restart_namestore ();
-}
-
-
 /* end of gnunet-setup-namestore-config.c */
