@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     (C) 2010, 2011 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2010, 2011 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 */
 
 /**
@@ -108,21 +108,11 @@ static const struct PlotInfo storage_data[] = {
  */
 static struct GNUNET_GTK_MainLoop *ml;
 
-/**
- * Should gnunet-peerinfo-gtk start in tray mode?
- */
-static int tray_only;
-
 /**
  * Handle to statistics subsystem.
  */
 static struct GNUNET_STATISTICS_Handle *statistics;
 
-/**
- * Task that refreshes connection graphic.
- */
-static GNUNET_SCHEDULER_TaskIdentifier connection_task;
-
 /**
  * When did this process start?
  */
@@ -136,20 +126,28 @@ static struct GNUNET_TIME_Absolute start_time;
  * @param subsystem name of subsystem that created the statistic
  * @param name the name of the datum
  * @param value the current value
- * @param is_persistent GNUNET_YES if the value is persistent, GNUNET_NO if not
- * @return GNUNET_OK to continue
+ * @param is_persistent #GNUNET_YES if the value is persistent, #GNUNET_NO if not
+ * @return #GNUNET_OK to continue
  */
 static int
-process_value_update (void *cls, const char *subsystem, const char *name,
-                      uint64_t value, int is_persistent)
+process_value_update (void *cls,
+                      const char *subsystem,
+                      const char *name,
+                      uint64_t value,
+                      int is_persistent)
 {
   GtkStatistics *stats = cls;
   char *id;
 
-  GNUNET_asprintf (&id, "%s: %s", subsystem, name);
-  gtk_statistics_update_value (stats, id,
+  GNUNET_asprintf (&id,
+                   "%s: %s",
+                   subsystem,
+                   name);
+  gtk_statistics_update_value (stats,
+                               id,
                                GNUNET_TIME_absolute_get_duration
-                               (start_time).rel_value_us / 1000000LL, value);
+                               (start_time).rel_value_us / 1000000LL,
+                               value);
   GNUNET_free (id);
   return GNUNET_OK;
 }
@@ -175,49 +173,55 @@ get_object (const char *name)
  * @param info what to draw
  */
 static void
-create_plot (const char *box_name, const struct PlotInfo *info)
+create_plot (const char *box_name,
+             const struct PlotInfo *info)
 {
   GtkBox *box;
   GtkStatistics *ret;
-  unsigned int i;
   char *id;
 
   ret = GTK_STATISTICS (gtk_statistics_new ());
   box = GTK_BOX (get_object (box_name));
 
-  for (i = 0; NULL != info[i].subsystem; i++)
+  for (unsigned int i = 0; NULL != info[i].subsystem; i++)
   {
-    GNUNET_asprintf (&id, "%s: %s", info[i].subsystem, info[i].name);
-    gtk_statistics_add_line (ret, id, info[i].label, info[i].color_name);
+    GNUNET_asprintf (&id,
+                     "%s: %s",
+                     info[i].subsystem,
+                     info[i].name);
+    gtk_statistics_add_line (ret,
+                             id,
+                             info[i].label,
+                             info[i].color_name);
     GNUNET_free (id);
-    GNUNET_STATISTICS_watch (statistics, info[i].subsystem, info[i].name,
-                             &process_value_update, ret);
+    GNUNET_STATISTICS_watch (statistics,
+                             info[i].subsystem,
+                             info[i].name,
+                             &process_value_update,
+                             ret);
   }
   gtk_widget_show (GTK_WIDGET (ret));
-  gtk_box_pack_start (box, GTK_WIDGET (ret), TRUE, TRUE, 0);
+  gtk_box_pack_start (box,
+                      GTK_WIDGET (ret),
+                      TRUE,
+                      TRUE,
+                      0);
 }
 
 
-
 /**
  * Task run on shutdown.
  *
  * @param cls unused
- * @param tc scheduler context, unused
  */
 static void
-shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
+shutdown_task (void *cls)
 {
-  GNUNET_GTK_tray_icon_destroy ();
   GNUNET_GTK_main_loop_quit (ml);
   ml = NULL;
-  GNUNET_STATISTICS_destroy (statistics, GNUNET_NO);
+  GNUNET_STATISTICS_destroy (statistics,
+                             GNUNET_NO);
   statistics = NULL;
-  if (connection_task != GNUNET_SCHEDULER_NO_TASK)
-  {
-    GNUNET_SCHEDULER_cancel (connection_task);
-    connection_task = GNUNET_SCHEDULER_NO_TASK;
-  }
 }
 
 
@@ -225,7 +229,8 @@ shutdown_task (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
  * Callback invoked if the application is supposed to exit.
  */
 void
-GNUNET_STATISTICS_GTK_quit_cb (GObject * object, gpointer user_data)
+GNUNET_STATISTICS_GTK_quit_cb (GObject *object,
+                               gpointer user_data)
 {
   GNUNET_SCHEDULER_shutdown ();
 }
@@ -236,19 +241,21 @@ GNUNET_STATISTICS_GTK_quit_cb (GObject * object, gpointer user_data)
  * is initialized.  Initializes up GTK and Glade.
  */
 static void
-run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
+run (void *cls)
 {
   GtkWidget *main_window;
 
   ml = cls;
 
-  if (GNUNET_OK != GNUNET_GTK_main_loop_build_window (ml, NULL))
+  if (GNUNET_OK !=
+      GNUNET_GTK_main_loop_build_window (ml,
+                                         NULL))
   {
     return;
   }
 
-  statistics =
-      GNUNET_STATISTICS_create ("gnunet-statistics-gtk",
+  statistics
+    = GNUNET_STATISTICS_create ("gnunet-statistics-gtk",
                                 GNUNET_GTK_main_loop_get_configuration (ml));
   if (NULL == statistics)
   {
@@ -264,42 +271,36 @@ run (void *cls, const struct GNUNET_SCHEDULER_TaskContext *tc)
   main_window = GNUNET_GTK_plug_me ("GNUNET_STATISTICS_GTK_PLUG",
                                     main_window);
   gtk_window_maximize (GTK_WINDOW (main_window));
-  if (NULL == getenv ("GNUNET_STATISTICS_GTK_PLUG"))
-    GNUNET_GTK_tray_icon_create (ml,
-                                 GTK_WINDOW (main_window),
-                                 "gnunet-gtk" /* FIXME: different icon? */ ,
-                                 "gnunet-statistics-gtk");
-
-  create_plot ("GNUNET_STATISTICS_GTK_connectivity_box", connection_data);
-  create_plot ("GNUNET_STATISTICS_GTK_traffic_box", traffic_data);
-  create_plot ("GNUNET_STATISTICS_GTK_storage_box", storage_data);
+  create_plot ("GNUNET_STATISTICS_GTK_connectivity_box",
+               connection_data);
+  create_plot ("GNUNET_STATISTICS_GTK_traffic_box",
+               traffic_data);
+  create_plot ("GNUNET_STATISTICS_GTK_storage_box",
+               storage_data);
 
   /* make GUI visible */
-  if (!tray_only)
-  {
-    gtk_widget_show (main_window);
-    gtk_window_present (GTK_WINDOW (main_window));
-  }
-  GNUNET_SCHEDULER_add_delayed (GNUNET_TIME_UNIT_FOREVER_REL,
-				&shutdown_task, NULL);
+  gtk_widget_show (main_window);
+  gtk_window_present (GTK_WINDOW (main_window));
+  GNUNET_SCHEDULER_add_shutdown (&shutdown_task,
+                                 NULL);
 }
 
 
 int
-main (int argc, char *const *argv)
+main (int argc,
+      char *const *argv)
 {
   static struct GNUNET_GETOPT_CommandLineOption options[] = {
-    {'t', "tray", NULL,
-     gettext_noop ("start in tray mode"), 0,
-     &GNUNET_GETOPT_set_one, &tray_only},
     GNUNET_GETOPT_OPTION_END
   };
 
   start_time = GNUNET_TIME_absolute_get ();
   if (GNUNET_OK !=
       GNUNET_GTK_main_loop_start ("gnunet-statistics-gtk",
-                                  "GTK GUI for viewing GNUnet statistics", argc,
-                                  argv, options,
+                                  "GTK GUI for viewing GNUnet statistics",
+                                  argc,
+                                  argv,
+                                  options,
                                   "gnunet_statistics_gtk_main_window.glade",
                                   &run))
     return 1;
