@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet
-     (C) 2011 Christian Grothoff (and other contributing authors)
+     Copyright (C) 2011 GNUnet e.V.
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -14,8 +14,8 @@
 
      You should have received a copy of the GNU General Public License
      along with GNUnet; see the file COPYING.  If not, write to the
-     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-     Boston, MA 02111-1307, USA.
+     Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+     Boston, MA 02110-1301, USA.
 
 */
 
@@ -122,10 +122,14 @@ static void
 gtk_statistics_finalize (GObject * object);
 
 
-G_DEFINE_TYPE (GtkStatistics, gtk_statistics, GTK_TYPE_WIDGET)
+G_DEFINE_TYPE_EXTENDED (GtkStatistics, gtk_statistics, GTK_TYPE_WIDGET, 0,
+                        G_ADD_PRIVATE (GtkStatistics))
+
+
 #if GTK_MAJOR_VERSION < 3
-  static gboolean statistics_expose (GtkWidget * statistics,
-                                     GdkEventExpose * event)
+static gboolean
+statistics_expose (GtkWidget * statistics,
+                   GdkEventExpose * event)
 {
   GtkAllocation alloc;
   cairo_t *cr;
@@ -161,8 +165,6 @@ gtk_statistics_class_init (GtkStatisticsClass * class)
 #if GTK_MAJOR_VERSION >= 3
   widget_class->draw = gtk_statistics_draw;
 #endif
-
-  g_type_class_add_private (class, sizeof (GtkStatisticsPrivate));
 }
 
 
@@ -217,15 +219,17 @@ gtk_statistics_new ()
  * @param color_name name of the color to use
  */
 void
-gtk_statistics_add_line (GtkStatistics * statistics, const char *id,
-                         const char *label, const char *color_name)
+gtk_statistics_add_line (GtkStatistics * statistics,
+			 const char *id,
+                         const char *label,
+			 const char *color_name)
 {
   GtkStatisticsPrivate *priv;
   struct ValueHistory *vh;
-  GdkColor c;
+  GdkRGBA c;
 
   g_return_if_fail (GTK_IS_STATISTICS (statistics));
-  g_return_if_fail (gdk_color_parse (color_name, &c));
+  g_return_if_fail (gdk_rgba_parse (&c, color_name));
   priv = statistics->priv;
   priv->values =
       g_realloc (priv->values,
@@ -234,9 +238,9 @@ gtk_statistics_add_line (GtkStatistics * statistics, const char *id,
       g_malloc (sizeof (struct ValueHistory));
   vh->id = strdup (id);
   vh->label = strdup (label);
-  vh->red = c.red / 65535.0;
-  vh->green = c.green / 65535.0;
-  vh->blue = c.blue / 65535.0;
+  vh->red = c.red;
+  vh->green = c.green;
+  vh->blue = c.blue;
   vh->last_history_offset = 0;
   vh->history_size = 0;
 }
